/****************************************************************************/
/*																			*/
/*				bruch.cpp Version 1.8.6										*/
/*																			*/
/*				Fachhochschule Jena - Informatik IIa						*/
/*				Projektarbeit Matrix										*/
/*				Set 2.08													*/
/*				bungsgruppe 1												*/
/*				Projektgruppe 3												*/
/*																			*/
/*				Jrgen Dffinger											*/
/*																			*/
/****************************************************************************/

/****************************************************************************/
/*						Beschreibung:										*/
/*																			*/
/*	Die Bruch.cpp enthlt den Code der Methoden der Klasse Bruch. Die		*/
/*	Klasse Bruch ermglicht es Zahlen als Bruch zu behandeln. Dabei werden	*/
/*	arithmetische Operatoren und Vergleichsoperatoren zur Verfgung			*/
/*	gestellt. Es werden nur Ganzzahlausdrcke fr Zhler und Nenner			*/
/*	verwendet, welche vom Typ long long sind. Ein Krzungsalgorithmus wurde	*/
/*	ebenfalls implementiert. Dieser kann nicht selber aufgerufen werden.	*/
/*	Das Krzen wird automatisch vorgenommen. Sobald Werte des Bruchs		*/
/*	verndert werden.														*/
/*																			*/
/****************************************************************************/



#ifndef BRUCH_CPP_INCLUDED
#define BRUCH_CPP_INCLUDED



#include <iostream>
#include "Bruch.h"



using namespace std;



	//	Initialisierung
	void Bruch::init()
	{
		//	Prfen das Nenner ob Null ist
		if (nenner == 0) throw "Konstruktorfehler (Nenner darf nicht 0 sein.)";

		//	grten gemeinsame Teiler (ggT) bilden
		base_type ggt;
		ggt = ggT(zaehler, nenner);

		//	Bruch mithilfe des ggT krzen
		zaehler /= ggt;
		nenner /= ggt;
		
		//	Minus gegebenenfalls vom Nenner in den Zhler verschieben
		setNenner();
	}



	//	Methode zur Bedingung Nenner nicht negativ
	void Bruch::setNenner() {if (nenner < 0) {nenner *= (-1); zaehler *= (-1);}}



	// grter gemeinsamer Teiler (ggT) (Rekursiv nach Euklidischen Algorithmus)
	Bruch::base_type Bruch::ggT(base_type a, base_type b) {return ((a % b) != 0) ? ggT(b, (a % b)) : b;}



	//	Bruch krzen
	Bruch::my_type Bruch::kuerzen() 
	{
		//	ggT bestimmen
		Bruch::base_type ggt(ggT(zaehler, nenner));

		//	Bruch mittels ggT krzen
		Bruch::my_type tmp(zaehler / ggt, nenner / ggt);

		//	mgliches Minus im Nenner in den Zhler verschieben
		tmp.setNenner();

		return tmp;
	}


	//	Standard-Konstruktor (Bruch wird auf 0 initialisiert)
	Bruch::Bruch()  : zaehler(0), nenner(1) {}



	//	Konstruktor zur bernahme eines Ganzzahlausdrucks.
	Bruch::Bruch(base_type gz)  : zaehler(gz), nenner(1) {init();}



	//	Konstruktor mit bergebenen Vorgabewerten fr Zaehler und Nenner
	Bruch::Bruch(base_type z, base_type n) : zaehler(z), nenner(n) {init();}



	// Copy-Konstruktor
	Bruch::Bruch(const my_type& orig) : zaehler(orig.zaehler), nenner(orig.nenner) {init();}	



	//	Destruktor
	Bruch::~Bruch(){}



	//	Eingabe
	istream& operator >>(istream &is, Bruch::my_type &z)
	{
		char c[1];
		is >> z.zaehler >> c[0];
		if (!(c[0] == '/')) 
			throw "Eingabefehler (Bruchzahl)";
		else 
			is >> z.nenner;
		if (z.nenner == 0) throw "Eingabefehler (Nenner darf nicht 0 sein.)";
		return is;
	}



	//	Ausgabe
	ostream& operator <<(ostream &os, const Bruch::my_type &b)
	{
		if (b.zaehler == 0)	
			os <<  "0"; 
		else if (b.nenner == 1)
			os << b.zaehler;
		else
			os << b.zaehler << "/" << b.nenner;
		return os;
	}



	//	Addition
	Bruch::my_type Bruch::operator + (const Bruch::my_type& b) const
	{return (Bruch::my_type ((b.nenner * zaehler) + (b.zaehler * nenner), b.nenner * nenner).kuerzen());}



	//	Addition mit Operator +=
	Bruch::my_type& Bruch::operator += (const Bruch::my_type& b) 
	{
		(*this) = (*this) + b;
		return (*this);
	}



	//	Subtraktion
	Bruch::my_type Bruch::operator - (const Bruch::my_type& b) const
	{return (Bruch::my_type((b.nenner * zaehler) - (b.zaehler * nenner), b.nenner * nenner).kuerzen());}



	//	Subtraktion mit Operator +=
	Bruch::my_type& Bruch::operator -= (const Bruch::my_type& b) 
	{
		(*this) = (*this) - b;
		return (*this);
	}



	// Multiplikation
	Bruch::my_type Bruch::operator * (const Bruch::my_type& b) const
	{return (Bruch::my_type(zaehler * b.zaehler, nenner * b.nenner).kuerzen());}



	// Multiplikation mit Operator *=
	Bruch::my_type& Bruch::operator *= (const Bruch::my_type& b)
	{
		(*this) = (*this) * b;
		return (*this);
	}



	//	Division 
	Bruch::my_type Bruch::operator / (const Bruch::my_type& b) const
	{
		//	Bedingung Division nicht durch 0 prfen (mathematischer Fehler)
		if (b.zaehler == 0) throw "mathematischer Fehler -> Division durch 0 (Bruchzahl)";

		//	Deklaration
		Bruch::my_type tmp;

		//	Division mittels Multiplikation, durch das Reziprok des Divisors
		tmp.zaehler = zaehler * b.nenner;
		tmp.nenner = nenner * b.zaehler;
		
		// Bruch krzen
		tmp = tmp.kuerzen();
		return tmp;
	}



	//	Division mit operator /=
	Bruch::my_type& Bruch::operator /= (Bruch::my_type& b)
	{
		(*this) = (*this) / b;
		return b;
	}



	// unres Minus
	Bruch::my_type Bruch::operator - () {return (Bruch::my_type ((*this) * Bruch::my_type(-1,1)));}



	//	kleiner als
	bool Bruch::operator < (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) < ((double)b.zaehler / (double)b.nenner)));}



	//	grer als
	bool Bruch::operator > (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) > ((double)b.zaehler / (double)b.nenner)));}



	//	kleiner gleich
	bool Bruch::operator <= (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) <= ((double)b.zaehler / (double)b.nenner)));}



	//	grer gleich
	bool Bruch::operator >= (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) >= ((double)b.zaehler / (double)b.nenner)));}



	//	gleich
	bool Bruch::operator == (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) == ((double)b.zaehler / (double)b.nenner)));}



	//	ungleich
	bool Bruch::operator != (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) != ((double)b.zaehler / (double)b.nenner)));}


	
	//	Zuweisungsoperator
	Bruch::my_type Bruch::operator = (Bruch::my_type& b) 
	{
		zaehler = b.zaehler;
		nenner = b.nenner;
		return (*this);
	}



	//	Gibt den Wert des Zhlers zurck
	Bruch::base_type Bruch::Zaehler() { return base_type(zaehler); }



	//	Gibt den Wert des Nenners zurck
	Bruch::base_type Bruch::Nenner() { return base_type(nenner); }

#endif