/****************************************************************************/
/*																			*/
/*				Matrix.h Version 1.8.6										*/
/*																			*/
/*				Fachhochschule Jena - Informatik IIa						*/
/*				Projektarbeit Matrix										*/
/*				Set 2.08													*/
/*				bungsgruppe 1												*/
/*				Projektgruppe 3												*/
/*																			*/
/*				Jrgen Dffinger											*/
/*																			*/
/****************************************************************************/

/****************************************************************************/
/*						Beschreibung:										*/
/*																			*/
/*	Die Matrix.h enthlt die Klasse Matrix. Diese Klasse ist eine			*/
/*	abgeleitete Klasse der Klassen MatrixArithmetic und MatrixMath.			*/
/*	Sie enthlt den Gleichungslser und dient dem Anwender als Zugang,		*/
/*	zu smtlichen Matrix-Operationen.										*/
/*																			*/
/****************************************************************************/



#ifndef MATRIX_H_INCLUDED
#define MATRIX_H_INCLUDED



//	Exceptions
//	Fr die Klasse Matrix sind die Fehlernummern 400 bis 499 vorgesehen
#define msg400	"Die Matrix mit den absoluten Gliedern ist kein Spaltenvektor."
#define msg401	"Die Ergebnismatrix ist kein Spaltenvektor."
#define msg402	"Die Anzahl absoluter Glieder stimmt nicht mit der Zeilenanzahl des Loesungsvektors ueberein."
#define msg403	"Die Anzahl der absoluten Glieder stimmt nicht mit der Anzahl Gleichungen in der Koeffizientenmatrix ueberein."



#include "MatrixArithmetic.h"
#include "MatrixMath.h"
#include "MatrixException.h"
#include <math.h>



using namespace std;



template<class T>
class Matrix : public MatrixArithmetic<T> , public MatrixMath<T>
{

private:

	//	Typendefinition
	typedef vector<T> base_vector;
	typedef vector<base_vector> base_type;
	typedef Matrix<T> my_type;
	typedef	unsigned int size_type;



	//	obere Dreiecksmatrix bilden
	//	dient als Hilfsfunktion beim Gleichungslser und wandelt die Matrix (A) in eine obere Dreiecksmatrix um.
	//	Dabei wird der jeweilige Faktor auf den Vektor mit den absoluten Gliedern (absVec) angewendet
	void toUpper(base_vector& absVec) 
	{
		//	Deklaration
		size_type rs = getRows();		//	Anzahl der Zeilen
		size_type cs = getColumns();	//	Anzahl Spalten
		T f;							//	Faktor zur Bildung der Zeilen
		T ref(0);						//	Referenzwert fr Prfungen
		bool swap;						//	Merkt sich fr die Folgeoperationen, ob ein Zeilentausch stattgefunden hat
		unsigned int i;					//	Zhlvariable
		
		//	Bilden der oberen Dreiecksmatrix
		for(unsigned r=1; r<=rs; r++)				// Hauptschleife um alle Zeilen zu erfassen
		{
			for(unsigned rr=(r+1); rr<=rs; rr++)	//	Geht die Zeilen durch, welche verndert werden mssen
			{
				swap = true;
				if ((*this)(r,r) == ref)			//	Wenn zum bilden des Faktors eine Division durch Null auftaucht
				{
					swap = false;
					for (i=rr; i<=rs; i++)			//	wird nach einer Zeile zum tauschen gesucht
						if ((*this)(i,r) != ref) {swap = true; break;}		// Ist eine vorhanden, dann werden die Zeilen getauscht, 							
					if (swap)												//ansonsten bleibt alles so, da alle Werte der Spalte eine Null haben.
					{							
						this->swapRow(r,i);		// Zeilen der Koeffizientenmatrix tauschen
						//	Tauschen der Zeilen des Vektors mit den absoluten Gliedern
						f = absVec[r-1];
						absVec[r-1] = absVec[i-1];
						absVec[i-1] = f;
					}
				}
				if (swap)	//	Fand ein Tausch statt oder war der Divisor fr die Faktorbildung nicht Null,
				{			//	dann Faktor und Zeilen berechnen
					f = (*this)(rr,r) / (*this)(r,r);			//	Faktor bilden
					absVec[rr-1] -= f * absVec[r-1];			//	Absolutvektorelement berechnen
					for(unsigned c=1; c<=rs; c++)
					{
						(*this)(rr,c) = (*this)(rr,c) - (f * (*this)(r,c));	//	Koeffizientenmatrixelement berechnen
					}
				}
			}
		}
	}



 	//	Prft ob es einen Wiederspruch zwischen der Koeffizientenmatrix und 
	//	dem Absulutgliedvektor gibt.
	bool isContradiction(const base_vector& absVec) 
	{
		//	Deklaration
		size_type rs = getRows();
		size_type cs = getColumns();
		T ref = 0;
		bool weiter;

		//	Zeilen durchgehen ob es einen Wiederspruch gibt
		for (unsigned r=1; r<=rs; r++)
		{
			weiter = true;
			for (unsigned c=1; c<=cs; c++)
			{
				if ((*this)(r,c) != ref) //	Wenn ein Zeilenelement nicht gleich Null ist,
				{
					weiter = false;		//	dann Schleife beenden, da kein Wiederspruch vorliegen kann
					break;				// ansonsten liegen in der Zeile nur Nullen vor
				}
			}
			if (weiter) if (absVec[r-1] != ref) return true;	//	Waren alle Zeilenelemente Nullen und ist das absolute Glied keine Null, dann liegt ein Wiederspruch vor.
		}
		return false;
	}



public:

	//	Konstruktoren
	Matrix() : MatrixVector() {}													//	keine Parameterbergabe, 3x3 Matrix wird erzeugt
	Matrix(size_type dim) : MatrixVector(dim) {}									// ein Parameter wird bergeben, dim x dim Matrix wird erzeugt
	Matrix(size_type Rows, size_type Columns) : MatrixVector(Rows, Columns) {}		// erzeugt Rows x Columns Matrix
	Matrix(const my_type& orig)	{ init(orig, orig.getRows(), orig.getColumns()); }	// Copy-Konstruktor


	//	Destruktor
	virtual ~Matrix(){}



	//	Gleichungslser nach Gau-Jordan fr lineare Gleichungssysteme 
	//	Dabei ist die Matrix so zu gestalten, dass Sie die Koeffizientenmatrix darstellt
	//	Die im Parameter zu bergebende Matrix ist als Spaltenvektor zu bergeben, welche
	//	die absoluten Glieder enthlt (AbsolutVektor). Der rechte Parameter stellt
	//	den Lsungsvektor dar, welcher von derselben Gre wie der Vektor mit den
	//	absoluten Gliedern sein muss.
	unsigned short solve(my_type& aV, my_type& ans) 
	{
		//	Bedingung prfen, dass Absolutvektor und Lsungsvektor nur aus einer Spalte bestehen
		if (aV.getColumns() != 1) throw MatrixException(400,msg400);
		if (ans.getColumns() != 1) throw MatrixException(401,msg401);

		//	Bedingung prfen, dass gleiche Anzahl absolute Glieder wie Variablen im Lsungsvektor vorliegen
		if (aV.getRows() != ans.getRows()) throw MatrixException(402,msg402);

		//	Deklaration
		size_type rs = getRows();		//	Anzahl Zeilen der Koeffizientenmatrix
		size_type cs = getColumns();	//	Anzahl Spalten der Koeffizientenmatrix

		// Bedingung prfen, dass die Koeffizientenanzahl mit denen der absoluten Glieder bereinstimmen,
		//	also Anzahl Zeilen bereinstimmen
		if (aV.getRows() != rs) throw MatrixException(403,msg403);

		//	weitere Deklarationen
		base_vector absVec;			//	Vektor mit den absoluten Gliedern
		my_type A(rs,cs);			//	Kopie der Koeffizientenmatrix
		T ref(0);					//	Referenzwert fr Prfungen
		T det(1);					//	Variable fr Determinantenwert

		A = (*this);
		absVec = aV.getColumnElements(1);

		//	obere Dreiecksmatrix bilden und Vektor mit absoluten Gliedern entsprechend anpassen
		A.toUpper(absVec);

		//	Prfen auf Lsbarkeit
		//	det(A) nicht gleich 0 dann liegt genau eine Lsunge vor,
		//	Determinante wird dadurch gebildet, 
		//	dass die Diagonalelemente der oberen Dreiecksmatrix multipliziert werden
		det = A(1,1);
		for(unsigned r=2; r<=rs; r++)
		{
			det *= A(r,r);
			if (det == ref) break;
		}
		if (det == ref) 
			if (A.isContradiction(absVec)) return 0; else return 2;
		
		//	Lsungsvektor bilden, durch Umwandlung der Koeffizientenmatrix (als obere Dreiecksmatrix)
		// in eine Einheitsmatrix
		size_type rc = rs;
		for(unsigned c=cs; c>0; c--)
		{
			if ((absVec[rc-1] != ref) || (A(rc,c) != ref))		
			{
				absVec[rc-1] /= A(rc,c);
				A(rc,c) /= A(rc,c);
				for(unsigned r=rc-1; r>0; r--)
				{
					A(r,c) = (A(r,c) * absVec[rc-1]);
					absVec[r-1] -= A(r,c);
					A(r,c) -= A(r,c);
				}
			}
			rc--;
		}

		//	Rckgabematrix anpassen und Werte der Lsung bergeben
		for (unsigned r=1; r<=rs; r++) ans(r,1) = absVec[r-1];
		return 1;
	}



	//	Zuweisungsoperator MatrixAritmetic
	MatrixArithmetic& operator = (MatrixArithmetic& m)
	{
		this->MatrixArithmetic::operator=(m);
		return (*this);
	}

	//	Zuweisungsoperator MatrixMath
	MatrixMath& operator = (MatrixMath& m)
	{
		this->MatrixMath::operator=(m);
		return (*this);
	}
};


#endif