/****************************************************************************/
/*																			*/
/*				MatrixArithmetic.h Version 1.8.6							*/
/*																			*/
/*				Fachhochschule Jena - Informatik IIa						*/
/*				Projektarbeit Matrix										*/
/*				Set 2.08													*/
/*				bungsgruppe 1												*/
/*				Projektgruppe 3												*/
/*																			*/
/*				Hannes Kuschick												*/
/*																			*/
/****************************************************************************/

/****************************************************************************/
/*						Beschreibung:										*/
/*																			*/
/*																			*/	
/*	Die MatrixArithmetic.h enthlt die Klasse MatrixArithmetic. Diese		*/
/*	Klasse ist eine abgeleitete Klasse der Klasse MatrixVector. Sie enthlt	*/
/*	folgende arithmetischen Methoden:										*/									
/*																			*/
/*	- Addition																*/
/*	- Subtraktion															*/
/*	- Multiplikation (Matrix, Vektor, Skalar)								*/
/*																			*/
/****************************************************************************/



#ifndef MATRIXARITHMETIC_H_INCLUDED
#define MATRIXARITHMETIC_H_INCLUDED



//	Exceptions
//	Fr die Klasse MatrixArithmetic sind die Fehlernummern 200 bis 299 vorgesehen
#define msg200	"Die Spaltenanzahl der Matrix 1 stimmt nicht mit der Zeilenanzahl der Matrix 2 ueberein."
#define msg201	"Bei den Spaltenvektoren liegt keine gleiche Anzahl an Zeilen vor, daher kann das Vektorprodukt nicht gebildet werden."
#define msg202	"Die beiden zu addierenden Matrizen stimmen nicht in der Anzahl der Zeilen und Spalten ueberein."
#define msg203	"Die beiden zu subtrahierenden Matrizen stimmen nicht in der Anzahl der Zeilen und Spalten ueberein."
#define msg204	"Es wurde versucht ein Vektorprodukt zu berechnen, obwohl keine Spaltenvektoren vorlagen."



#include "MatrixVector.h"
#include "MatrixException.h"



using namespace std;



template<class T> class MatrixArithmetic : virtual public MatrixVector<T>
{

private:

	//	Typendefinition
	typedef vector<T> base_vector;
	typedef vector<base_vector> base_type;
	typedef MatrixArithmetic<T> my_type;	
	typedef	unsigned int size_type;



	// Prft ob die beiden angegebenen Matrizen vom selben Typ sind, also ob Zeilen- und Spaltenanzahl gleich sind.
	bool isSameType (const my_type& m) const
	{
		return ((m.getRows() != getRows()) || (m.getColumns() != getColumns())) ? false : true;
	}



	//	Vektorproduktberechnung aus zwei Spaltenvektoren
	my_type VectorProduct(my_type& m)
	{
		//	Bedingung prfen ob es sich um Spaltenvektoren handelt
		if ((this->isColumnVector() != true) || (m.isColumnVector() != true)) throw MatrixException(204,msg204);

		//	Bedingung prfen ob Spaltenvektoren gleiche Anzahl Zeilen haben.
		if (m.getRows() != getRows()) throw MatrixException(201,msg201);

		//	Deklaration einer Ergebnismatrix
		my_type tmp(getRows(),1);
				
		//	Berechnung des Vektorproduktes (Kreuzproduktes)
		size_type rs = getRows();	//	Anzahl der Rechenoperationen
		unsigned i = 2;				
		unsigned j = 3;				
		unsigned r = 1;					
		
		//	Vektorprodukt bilden
		while(r<=rs)
		{
			tmp(r,1) = (*this)(i,1) * m(j,1) - (*this)(i,1) * m(j,1);
			i++;
			j++;
			r++;
			if (i > rs) i = 1;
			if (j > rs) j = 1;
		}
		return tmp;
	}



public:

	//	Konstruktoren
	MatrixArithmetic() : MatrixVector(){}										
	MatrixArithmetic(size_type dim) : MatrixVector(dim) {}
	MatrixArithmetic(size_type Rows, size_type Columns) : MatrixVector(Rows, Columns) {}
	MatrixArithmetic(const my_type& orig) { init(orig, orig.getRows(), orig.getColumns()); }



	//	Destruktor
	virtual ~MatrixArithmetic(){}



	//	unres +
	my_type& operator +() const {return (*this);}



	//	unres -
	my_type& operator -() const
	{
		// Deklaration mit Zuweisung
		size_type rs = getRows();
		size_type cs = getColumns();
		my_type tmp(rs,cs);

		//	unres Minus durchfhren
		for (unsigned r=1; r<=rows; r++)
			for(unsigned c=1; c<=columns; c++)
				tmp(r,c) = -(*this)(r,c);

		return tmp;
	}



	// Operator + berladen
	my_type operator +(my_type& m) 
	{
		// Deklaration
		size_type rows = getRows();
		size_type columns = getColumns();

		//	Bedingungen gleicher Matrizen prfen
		if (this->isSameType(m) != true) throw MatrixException(202,msg202);

		// Anlegen einer temporren Matrix
		my_type tmp(rows, columns);

		// Addition durchfhren
		for (unsigned r=1; r<=rows; r++)
			for(unsigned c=1; c<=columns; c++)
				tmp(r,c) = (*this)(r,c) + m(r,c);

		return tmp;
	}



	// Operator - berladen
	my_type operator -(my_type& m) 
	{
		//	Bedingungen gleicher Matrizen prfen
		if (this->isSameType(m) != true) throw MatrixException(203,msg203);

		// Deklaration
		size_type rows = getRows();
		size_type columns = getColumns();
		my_type tmp(rows, columns);


		// Subtraktion durchfhren
		for (unsigned r=1; r<=rows; r++)
			for(unsigned c=1; c<=columns; c++)
				tmp(r,c) = (*this)(r,c) - m(r,c);

		return tmp;
	}



	//	Addition mit += Operator
	my_type& operator += (my_type& m) 
	{
		(*this) = (*this) + m;
		return (*this);
	}

	

	//	Subtraktion mit -= - Operator
	my_type& operator -= (my_type& m) 
	{
		(*this) = (*this) - m;
		return (*this);
	}



	//	Matrix-Multiplikation nach Falk-Schema
	my_type operator *(my_type& m1) 
	{
		//	Prfen ob Spaltenvektoren vorliegen und 
		//	weiterleiten zur Berechnung des Vektorproduktes (Kreuzproduktes)
		if ((this->isColumnVector()) && (m1.isColumnVector())) return this->VectorProduct(m1);

		//	Deklaration
		size_type m = getRows();		//	Zeilenanzahl Matrix 1
		size_type n = getColumns();		//	Spaltenanzahl Matrix 1
		size_type p = m1.getColumns();	//	Spaltenanzahl Matrix 2

		//	Bedingung gleicher Anzahl Spalten der Matrix 1 und Zeilenanzahl der Matrix 2 prfen
		if (n != m1.getRows()) throw MatrixException(200,msg200);

		//	Anlegen einer temporren Matrix
		my_type tmp((m),(p));

		// Berechnung des Multiplikationsprodukts
		for(unsigned mu=1; mu<=m; mu++)
			for(unsigned la=1; la<=p; la++)
				for(unsigned vu=1; vu<=n; vu++)
					tmp(mu,la) += (*this)(mu,vu) * m1(vu,la);
		return tmp;

	}



	//	Matrix-Multiplikation mit Operator *=
	my_type& operator *= (my_type& m) 
	{
		(*this) = (*this) * m;
		return (*this);
	}



	// Multiplikation mit einem Skalar
	my_type operator * (const T& val)  
	{
		//	Deklaration
		size_type rs = getRows();
		size_type cs = getColumns();
		my_type tmp(rs,cs);
		tmp = (*this);

		//	Berechnung des Multiplikationsproduktes
		for(unsigned r=1; r<=rs; r++)
			for(unsigned c=1; c<=cs; c++)
				tmp(r,c) *= val;
		return tmp;
	}



	//	Multiplikation mit einem Skalar und Operator *=
	my_type& operator *= (const T& val) 
	{
		(*this) = (*this) * val;
		return (*this);
	}



	//	Zuweisungsoperator
	MatrixVector& operator = (MatrixVector& m)
	{
		this->MatrixVector::operator=(m);
		return (*this);
	}

};



// Multiplikation in der Form Skalar * Matrix
template <class T> MatrixArithmetic<T> operator*(T& f, MatrixArithmetic<T>& m) 
{
	unsigned int rs = m.getRows();
	unsigned int cs = m.getColumns();
	MatrixArithmetic<T> tmp(rs,cs);
	for (unsigned r=1; r<=rs; r++)
		for (unsigned c=1; c<=cs; c++)
			tmp(r,c) = m(r,c) * f;
	return tmp;
}



#endif