/****************************************************************************/
/*																			*/
/*				MatrixVector.h Version 1.8.6								*/
/*																			*/
/*				Fachhochschule Jena - Informatik IIa						*/
/*				Projektarbeit Matrix										*/
/*				Set 2.08													*/
/*				Projektgruppe 1												*/
/*																			*/
/*				Florian Hilbrecht											*/
/*																			*/
/****************************************************************************/

/****************************************************************************/
/*						Beschreibung:										*/
/*																			*/
/*																			*/	
/*	Die MatrixVector.h enthlt die Klasse MatrixVector. Diese Klasse ist	*/
/*	die Basisklasse aller weiteren Matrix-Klassen. Sie stellt Methoden zur	*/
/*	Ein- und Ausgabe, sowie zum Abfragen von Eigenschaften zur Verfgung.	*/
/*	Des Weiteren enthlt sie das Attribut elements. Dieses ist vom Type		*/
/*	vector<vector<T>> und enthlt die Werte der Matrix.						*/
/*																			*/
/****************************************************************************/



#ifndef MATRIXVECTOR_H_INCLUDED
#define MATRIXVECTOR_H_INCLUDED



//	Exceptions
//	Fr die Klasse MatrixVector sind die Fehlernummern 100 bis 199 vorgesehen
#define msg100	"Die Angabe der Zeilenanzahl und/oder Spaltenanzahl ist kleiner als 1."
#define	msg101	"Sie haben eine Zeilenanzahl kleiner 1 eingegeben."
#define	msg102	"Sie haben eine Spaltenanzahl kleiner 1 eingegeben."
#define msg103	"Zeilen- und/oder Spaltenangabe lag ausserhalb des Definitionsbereiches der Matrix."
#define msg104	"Die Zeilen- und Spaltenanzahl der Matrizen stimmen nicht ueberein."
#define msg105	"Die Zeilenangabe lag ausserhalb des Definitionsbereiches der Matrix."
#define msg106	"Die Spaltenangabe lag ausserhalb des Definitionsbereiches der Matrix."



#include <vector>
#include <iostream>
#include <time.h>
#include "MatrixException.h"



using namespace std;



template<class T> class MatrixVector 
{

private:
	
	//	Typdefinitionen
	typedef vector<T> base_vector;
	typedef vector<base_vector> base_type;
	typedef MatrixVector<T> my_type;	
	typedef	unsigned int size_type;



	//	Attributdeklaration
	base_type elements;		//	Enthlt alle Elemente der Matrix
	size_type rows;			//	Zeilenanzahl der Matrix
	size_type columns;		//	Spaltenanzahl der Matrix



	// Indexoperator berladen zum Zugriff auf einzelne Zeilen der Matrix
	//	Gekapselt, da es sonst mglich wre einzelne Elemente, Zeilen oder Spalten mithilfe der
	//	Memberfunktion, der Klasse vector zu zerstren
	base_vector& operator[](size_type index) 
	{
		//	Bedingung prfen, dass der Indexparamter innerhalb des Definitionsbereiches der Matrix liegt
		if ((index<0) || (index>=rows)) throw MatrixException(103,msg103);

		//	Rckgabe des einzelnen Elements der Matrix
		return(elements[index]);
	}



public:

	// Konstruktoren
	MatrixVector() {resize(3,3);}															//	keine Parameterbergabe, 3x3 Matrix wird erzeugt
	MatrixVector(size_type dim) {resize(dim,dim);}											// ein Parameter wird bergeben, dim x dim Matrix wird erzeugt
	MatrixVector(size_type Rows, size_type Columns) {resize(Rows, Columns);}				// erzeugt Rows x Columns Matrix
	MatrixVector(const my_type& orig) { init(orig, orig.getRows(), orig.getColumns()); }	//	Copy - Konstruktor
	


	// Destruktor
	virtual ~MatrixVector(){}



	//	Initialisierungsroutine fr Copy-Konstruktor
	void init(const my_type& m, size_type Rows, size_type Columns)
	{
		elements = m.elements;
		rows = Rows;
		columns = Columns;
	}



	// Initialisierung der Matrix
	void resize(size_type Rows, size_type Columns)
	{
		if ((Rows<1) || (Columns<1)) throw MatrixException(100,msg100);	// Fehler 100 ausgeben, falls Zeilen und Spalten kleiner 1 sind
		
		elements.resize(Rows);	//	Zeilen initialisieren
		for(unsigned r=0; r<Rows; r++)	
			elements[r].resize(Columns);	//	Spalten der entsprechenden Zeile initialisieren
		
		// Merken der Anzahl von Zeilen und Spalten in den entsprechenden Attributen
		rows = Rows;	
		columns = Columns;
	}



	// Zeilenanzahl zurckgeben
	size_type getRows() const {return rows;}



	// Spaltenanzahl zurckgeben
	size_type getColumns() const {return columns;}



	// Gre der  Matrix in Form rows x columns zurckgeben
	size_type size() const {return rows*columns;}


	
	// Zeilenanzahl neu definieren (Achtung!!! Elemente der Matrix werden auf 0 initialisiert)
	void setRows(size_type Rows) 
	{
		(Rows<1) ? throw MatrixException(101,msg101) :	resize(Rows,columns);
	}



	// Spaltenanzahl neu definieren (Achtung!!! Elemente der Matrix werden auf 0 initialisiert)
	void setColumns(size_type Columns) 
	{
		(Columns<1) ? throw MatrixException(102,msg102) : resize(rows,Columns);
	}



	// shift-operator << berladen fr die Ausgabe der Matrix
	friend ostream& operator <<(ostream& os, my_type& m) 
	{
		//	Deklaration
		unsigned rs = m.getRows();
		unsigned cs = m.getColumns();

		//	Auslesen der Werte und dem Ausgabe-Stream bergeben
		for(unsigned r=1; r<=rs; r++) 
		{
			for(unsigned c=1; c<=cs; c++)
			{
				os << m(r,c) << "\t";
			}
			os << endl;
		}
		return os;
	}



	// shift-Operator >> berladen zur Eingabe einer Matrix
	friend istream& operator >>(istream& is, my_type& m) 
	{
		//	Deklaration
		unsigned rs = m.getRows();
		unsigned cs = m.getColumns();

		//	Auslesen der eingegebenen Werte und der Matrix bergeben
		for(unsigned r=1; r<=rs; r++) 
			for(unsigned c=1; c<=cs; c++)
				is >> m(r,c);
		return is;
	}



	// Indexoperator berladen zum Zugriff auf ein einzelnes Element der Matrix,
	//	dabei ist der row = 1 auch Zeile 1 der Matrix, sowie column = 1 auch
	//	Spalte 1 der Matrix
	T& operator()(size_type row, size_type column) 
	{
		//	Bedingung prfen, dass die Indexangaben innerhalb des Definitionsbereiches der Matrix liegen
		if ((row<1) || (row>rows) || (column<1) || (column>columns)) throw MatrixException(103,msg103);

		//	Rckgabe des einzelnen Matrix-Wertes
		return elements.at(--row).at(--column);
	}



	// berladen des Vergleichsoperators == 
	bool operator == (my_type& m)  
	{
		//	Bedingung prfen, dass beide Matrizen vom gleichen Typ sind
		if ((m.rows != rows) || (m.columns != columns)) return false;

		//	Vergleich durchfhren
		for (unsigned r=1; r<=rows; r++)
			for(unsigned c=1; c<=columns; c++)
				if (m(r,c) != (*this)(r,c)) return false;
		return true;
	}



	// berladen des Vergleichsoperators != 
	bool operator != (my_type& m)  
	{
		return ((*this) == m) ? false : true;
	}



	//	Gibt alle Elemente einer bestimmten Zeile als Zeilenvector zurck
	base_vector getRowElements(size_type index) const 
	{
		//	Bedingung prfen, dass der Indexparameter innherlab des Definitionsbereiches der Matrix liegt
		if ((index < 1) || (index>rows)) throw MatrixException(105,msg105);

		//	einzelne Zeile zurckgeben
		return elements[--index];
	}



	//	Gibt alle Elemente einer bestimmten Spalte als Spaltenvector zurck
	base_vector getColumnElements(size_type index) 
	{
		//	Bedingung prfen, dass der Indexparameter innerhalb des Definitionsbereiches der Matrix liegt
		if ((index < 1) || (index>columns)) throw MatrixException(106,msg106);

		//	Deklaration
		base_vector tmp;
		tmp.resize(rows);
		
		//	Spaltenelemente in einen Vector kopieren und zurckgeben
		for (unsigned r=1; r<=rows; r++)
			tmp[(r-1)] = (*this)(r,index);

		return tmp;
	}



	//	Prft ob es sich um eine quadratische Matrix handelt
	bool isSquare() const {return (rows == columns) ? true : false;}



	//	Prft ob es sich um eine Nullmatrixhandelt und 
	//	gibt entsprechenden true oder false zurck
	bool isZero() 
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (!isSquare()) return false;

		//	Deklaration des Referenzwertes fr die Prfung
		T ref = 0;

		//	Prfung auf Nullmatrix durchfhren
		for (unsigned r=1; r<=rows; r++)
			for (unsigned c=1; c<=columns; c++)
				if ((*this)(r,c) != ref) return false;
		return true;
	}



	//	Prft ob es sich bei der Matrix um eine Einheitsmatrix handelt.
	bool isUnit()
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (!isSquare()) return false;

		//	Deklaration der Referenzwerte fr die Prfung
		T ref = 0;
		T ref2 = 1;

		//	Prfung auf Einheitsmatrix durchfhren
		for (unsigned r=1; r<=rows; r++) {
			for (unsigned c=1; c<=columns; c++) {
				if (r==c) {	
					if ((*this)(r,c) != ref2) return false;	//	Bei gleicher Zeile und Spalte muss der Wert eine 1 sein
				} else {
					if ((*this)(r,c) != ref) return false;	//	Bei nichtgleicher Zeile und Spalte muss der Wert eine 0 sein
				}
			}
		}
		return true;
	}



	//	Prft ob es sich um eine Diagonalmatrix handelt
	bool isDiagonal() 
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (isSquare() != true) return false;

		//	Deklaration des Referenzwertes fr die Prfung
		T ref = 0;

		//	Prfung auf Diagonalmatrix durchfhren
		for(unsigned r=1; r<=rows; r++)
			for(unsigned c=1; c<=columns; c++)
				if (r != c)
					if ((*this)(r,c) != ref) //	Elemente auerhalb der Diagonale mssen Null sein
						return false;

		return true;
	}



	//	Prft ob es sich um eine Skalarmatrix handelt
	bool isScalar() 
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (isSquare() != true) return false;

		//	Deklaration der Referenzwerte fr die Prfung
		T ref = 0;
		T scalar = (*this)(1,1);

		//	Prfung auf Skalarmatrix durchfhren
		for(unsigned r=1; r<=rows; r++) 
		{
			for(unsigned c=1; c<=columns; c++) 
			{
				if (r != c) 
				{
					if ((*this)(r,c) != ref) return false;	//	Elemente auerhalb der Hauptdiagonalen mssen 0 sein
				} else {
					if ((*this)(r,c) != scalar) return false;	//	Elemente der Hauptdiagonalen mssen alle den gleichen Werrt haben
				}
			}
		}
		return true;
	}



	//	Prft ob es sich um eine rechte/obere Dreiecksmatrix handelt.
	bool isUpper()
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (isSquare() != true) return false;

		//	Deklaration des Referenzwertes fr die Prfung
		T ref = 0;

		//	Prfung auf obere Dreiecksmatrix durchfhren
		for(unsigned r=1; r<=rows; r++) 
			for(unsigned c=1; c<=columns; c++) 
				if ((r>c) && ((*this)(r,c) != ref)) return false;	
		return true;
	}



	//	Prft ob es sich um einer linke/untere Dreiecksmatrix
	bool isLower() 
	{
		//	Bedingung prfen, dass es sich um eine quadratische Matrix handelt
		if (isSquare() != true) return false;

		//	Deklaration des Referenzwertes fr die Prfung
		T ref = 0;
		
		//	Prfung auf untere Dreiecksmatrix durchfhren
		for(unsigned r=1; r<=rows; r++) 
			for(unsigned c=1; c<=columns; c++) 
				if ((r<c) && ((*this)(r,c) != ref)) return false;
		return true;
	}



	// Prft auf Dreiecksmatrix und gibt entsprechenden Zahlenwert zurck
	unsigned short isTriangular() 
	{
		if (isDiagonal()) return 3;		//	bei oberer und unterer Dreiecksmatrix (Diagonalmatrix) wird eine 3 zurckgegeben
		if (isUpper()) return 1;		//	bei oberer Dreiecksmatrix wird 1 zurckgegeben
		if (isLower()) return 2;		//	bei unterer Dreiecksmatrix wird 2 zurckgegeben
		return 0;						// handelt es sich nicht um eine Dreiecksmatrix wird eine 0 zurckgegeben
	}



	// Prft ob es sich um einen Spaltenvektor handelt
	bool isColumnVector() const {return ((columns == 1) && (rows > 1)) ? true : false;}



	//	Prft ob es sich um einen Zeilenvektor handelt
	bool isRowVector() const {return ((rows == 1) && (columns>1)) ? true : false;}



	//	Tauscht Zeilen aus
	void swapRow(size_type index1, size_type index2) 
	{
		//	Bedingung prfen, dass die Indexparameter innerhalb des Definitionsbereiches der Matrix liegen
		if ((index1>rows) || (index1<1) || (index2>rows) || (index2<1)) throw MatrixException(103,msg103);

		//	Anpassen der Indexoperatoren
		index1--;
		index2--;

		//	Deklaration einer temporren Matrix
		my_type tmp(rows, columns);

		// Tausch durchfhren
		tmp = (*this);
		tmp[index1] = (*this)[index2];
		tmp[index2] = (*this)[index1];

		(*this) = tmp;
	}



	//Tauscht Spalten aus
	void swapColumn(size_type index1, size_type index2) 
	{
		//	Bedingung prfen, dass die indexoperatoren innerhalb des Definitionsbereiches der Matrix liegen
		if ((index1>columns) || (index1<1) || (index2>columns) || (index2<1)) throw MatrixException(103,msg103);

		// Deklaration einer temporren Matrix
		my_type tmp(rows, columns);

		// Tausch durchfhren
		tmp = (*this);
		for (unsigned r=1; r<=rows; r++) {
			tmp(r,index1) = (*this)(r,index2);
			tmp(r,index2) = (*this)(r,index1);
		}

		(*this) = tmp;
	}



	//	Zuweisungsoperator
	my_type& operator = (const my_type& m) 
	{
		//	Prfen dass die Matrizen vom selben Typ sind
		if ((rows != m.getRows()) || (columns != m.getColumns())) throw MatrixException(104,msg104);

		//	Elemente der Matrix bergeben
		elements = m.elements;
		return (*this);
	}

};


#endif